"""
Separted classes and functions used by the availability module.

Note that these classes and variables are all imported higher up at the
top of the availability module. They can be referenced from there instead
of digging in deeper to these submodules.
"""
