"""
Separted classes and functions used by the conjunctions module.

Note that these classes and variables are all imported higher up at the
top of the conjunctions module. They can be referenced from there instead
of digging in deeper to these submodules.
"""
