"""
Separted classes and functions used by the api module.

Note that these classes and variables are all imported higher up at the
top of the api module. They can be referenced from there instead of digging
in deeper to these submodules.
"""
